package bowling;

import java.util.List;

class Frame {
    private static final int TOTAL_PINS = 10;

    private List<Frame> frames;

    protected Throw[] pinsDropped = new Throw[2];

    protected int throwsMadeInFrame;

    Frame(List<Frame> frames) {
        this.frames = frames;
        pinsDropped[0] = new Throw();
        pinsDropped[1] = new Throw();
    }

    boolean isFirstFrame() {
        return frames.indexOf(this) == 0;
    }

    boolean isLastFrame() {
        return frames.indexOf(this) == frames.size() - 1;
    }

    Frame nextFrame() {
        return frames.get(frames.indexOf(this) + 1);
    }

    Frame previousFrame() {
        return frames.get(frames.indexOf(this) - 1);
    }

    int index() {
        return frames.indexOf(this);
    }

    String[] displayPinsDropped() {
        String[] pinsDroppedAsStrings = new String[pinsDropped.length];
        for (int i = 0; i < pinsDropped.length; i++) {
            pinsDroppedAsStrings[i] = displayThrow(i, pinsDropped[i]);
        }

        return pinsDroppedAsStrings;
    }

    private String displayThrow(int throwIndex, Throw pinsDropped) {
        if (pinsDropped.intValue() == TOTAL_PINS) {
            return "X";
        } else if (isSpare() && throwIndex == 1) {
            return "/";
        } else {
            return pinsDropped.toString();
        }
    }

    void drop(int pinsDropped) {
        this.pinsDropped[throwsMadeInFrame++] = new Throw(pinsDropped);
    }

    int throwsMadeInFrame() {
        return throwsMadeInFrame;
    }

    boolean isComplete() {
        return throwsMadeInFrame() == requiredThrowsForFrame();
    }

    protected int requiredThrowsForFrame() {
        if (isStrike()) {
            return 1;
        } else {
            return 2;
        }
    }

    protected boolean isStrike() {
        return firstThrowPinsDropped() == TOTAL_PINS;
    }

    protected boolean isSpare() {
        return !isStrike() && firstThrowPinsDropped() + secondThrowPinsDropped() == TOTAL_PINS;
    }

    private int firstThrowPinsDropped() {
        return getThrow(0).intValue();
    }

    private int secondThrowPinsDropped() {
        return getThrow(1).intValue();
    }

    Throw score() {
        Throw scoreForCurrentFrame = scoreCurrentFrame();
        if (!isFirstFrame()) {
            scoreForCurrentFrame = previousFrame().score().add(scoreForCurrentFrame);
        }

        return scoreForCurrentFrame;
    }

    protected Throw scoreCurrentFrame() {
        Throw score = null;
        if (isSpare()) {
            score = new Throw(10).add(nextFrame().firstThrowForScoring());
        } else if (isStrike()) {
            score = new Throw(10).add(nextFrame().firstThrowForScoring()).add(nextFrame().secondThrowForScoring());
        } else {
            score = firstThrowForScoring().add(secondThrowForScoring());
        }
        return score;
    }

    private Throw firstThrowForScoring() {
        return getThrow(0);
    }

    protected Throw secondThrowForScoring() {
        if (isStrike()) {
            return nextFrame().firstThrowForScoring();
        }

        return getThrow(1);

    }

    protected Throw getThrow(int throwIndex) {
        return pinsDropped[throwIndex];
    }
}
